#include "pest.h"
#include "packet.h"
#include "peer.h"
#include "chain.h"
#include "crypto.h"
#include "net.h"
#include "io.h"
#include "log.h"
#include "time.h"
#include "util.h"
#include "config.h"
#include "state.h"

//TODO: brian name for this
bool pest_init(pestev_f func){
  char    path[PATH_BUF];
  size_t  cnt;

  pest_func = func;

  if(!knob_load()) return 0;
  dedup_load();

  FMT_PATH(path,"%s",DIR_MESS);
  mess_init(&mess);
  mess_path(&mess,path);
  if(!mess_load(&mess)){
    irclog_err("failed to load mess: '%s'",path);
    return 0;
  }

  FMT_PATH(path,"%s",DIR_CHAIN);
  if(!mkdir_p(path)) return 0;

  FMT_PATH(path,DIR_CHAIN"/%s",FILE_SELF);
  if(file_exists(path)){
    if(!hash_load(path,&self,&cnt,1)) return 0;
  }else{
    hash_zero(self);
    if(!hash_store(path,(hash_t const *)&self,1)) return 0;
  }

  FMT_PATH(path,DIR_CHAIN"/%s",FILE_NET);
  if(file_exists(path)){
    if(!hash_load(path,&net,&cnt,1)) return 0;
  }else{
    hash_zero(self);
    if(!hash_store(path,(hash_t const *)&net,1)) return 0;
  }

  console_stamp(0,mono_sec());

  return 1;
}

static void peer_check_active(uint64_t time){
  for(size_t x = 0;x < peer_cnt;x++){
    peer_t *p       = peer+x; 
    bool    active  = p->active;

    p->active = time < p->pak_stamp+irctime;
    if(pest_func && !active && p->active){
      pest_func(PEST_ACTIVE,p->handle[0],0);
    }
    if(pest_func && active && !p->active){
      pest_func(PEST_INACTIVE,p->handle[0],0);
    }
  }
}

void pest_tick(uint64_t time,uint64_t ntime,bool active){
  if(!active){
    udp_recv(0,0,pest_port,0,0);
    return;
  }

  dedup_clean         (time);
  hearsay_process     (ntime);
  pending_process     (ntime);
  getdata_process     (ntime);
  getdata_retry       (time);
  rekey_clean         (time);
  rekey_broadcast     (time);
  banner_broadcast    (time);
  addrcast_broadcast  (time);
  ignore_broadcast    (ntime);
  peer_check_active   (time);

  for(size_t x = 0;x < PEST_RECV_MAX;x++){
    if(!pest_recv(time)) break;
  }
}

bool pest_broadcast(char const *msg){
  return send_str(CMD_BROADCAST,msg,0);
}

bool pest_direct(char const *handle,char const *msg){
  peer_t *p = peer_lookup(handle);
  if(!p) return 0;
  if(!peer_cansend(p,1)) return 0;
  return send_str(CMD_DIRECT,msg,p);
}

bool pest_achtung(char const *msg){
  uint16_t peer_idx[peer_cnt];

  if(!index_shuffle(peer_idx,peer_cnt)){
    irclog_err("could not shuffle index, achtung not sent");
    return 0;
  }

  for(size_t x = 0;x < peer_cnt;x++){
    peer_t *p = peer+peer_idx[x];
    if(!peer_cansend(p,0)) continue;
    send_str(CMD_DIRECT,msg,p);
  }

  return 1;
}

static bool resolve(char const *handle,
                    char const *chname,
                    chain_t    *c,
                    ht_t       *head,
                    ht_t       *tail){

  hash_t *l;
  hash_t  h;
  hash_t  t;

  if(ht_forks (head,tail) == 0 &&
     ht_breaks(head,tail) == 0){
    return 0;
  }

  l = ht_last(head);
  if(!l){
    irc_err("make sure you have seen at least one "
            "message from peer before resolve fork/break");
    return 1;
  }
  hash_copy(h,*l);

  if(chain_resolve(c,h,t)){
    ht_clear(head,h);
    ht_clear(tail,t);
    irclog_out("peer '%s' %s fork/break resolved",
               handle,chname);
  }else{
    irclog_err("failed to resolve peer '%s' %s fork/break",
               handle,chname);
  }

  return 1;
}

void pest_resolve(char const *handle){
  size_t  n = 0;
  peer_t *p = peer_lookup(handle);
  if(!p) return;

  if(resolve(p->handle[0],"broadcast",
             &p->bchain,&p->bhead,&p->btail)){
    n++;
  }

  if(resolve(p->handle[0],"direct message",
             &p->dchain,&p->dhead,&p->dtail)){
    n++;
  }

  if(!n){
    irc_err("cannot resolve, peer '%s' chain is not forked/broken",
            p->handle[0]);
  }
}

