#include "crypto.h"
#include "log.h"
#include "state.h"
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>

#define RNG_BORKTEST_SIZE             256

bool rng_open(char const *path){
  uint8_t bork[RNG_BORKTEST_SIZE];

  rng_fd = open(path,O_RDONLY);
  if(rng_fd == -1){
    log_errno("open");
    irclog_err("could not open rng '%s'",path);
    return 0;
  }

  if(!rng_read(bork,sizeof(bork))){
    close(rng_fd);
    return 0;
  }

  if(entropy_missing(bork,sizeof(bork))){
    irclog_err("rng '%s' is borken, all bytes were 0x%02X",
               path,bork[0]);
    close(rng_fd);
    return 0;
  }

  return 1;
}

void rng_close(void){
  close(rng_fd);
  rng_fd = -1;
}

static ssize_t _rng_read(void *data,size_t size){
  memset_ffs(data,0x00,size);
  return read(rng_fd,data,size);
}

bool rng_read(void *data,size_t size){
  ssize_t r = _rng_read(data,size);
  if(r != (ssize_t)size){
    if(r == -1) log_errno("read");
    irclog_err("could not read %zu bytes from rng",size);
    return 0;
  }
  return 1;
}

bool rng_read_nolog(void *data,size_t size){
  return _rng_read(data,size) == (ssize_t)size;
}

bool entropy_missing(uint8_t const *data,size_t size){
  uint8_t first = data[0];
  for(size_t x = 1;x < size;x++){
    if(data[x] != first) return 0;
  }
  return 1;
}

