#define _XOPEN_SOURCE 600
#include "wipe.h"
#include "crypto.h"
#include "log.h"
#include "util.h"
#include "state.h"
#include <ftw.h>
#include <unistd.h>
#include <fcntl.h>

void state_zero(void){
  BUF_ZERO(data_path);
  BUF_ZERO(self_handle);
  VAR_ZERO(running);
  VAR_ZERO(scram);
  VAR_ZERO(mono_offset);
  BUF_ZERO(log_path);
  VAR_ZERO(log_fp);
  //VAR_ZERO(rng_fd);

  BUF_ZERO(irc_chan);
  BUF_ZERO(irc_host);
  VAR_ZERO(irc_conned);
  VAR_ZERO(irc_usered);
  VAR_ZERO(irc_passed);
  VAR_ZERO(irc_nicked);
  VAR_ZERO(irc_joined);
  VAR_ZERO(irc_ready);

  VAR_ZERO(udp_listen_fd);
  VAR_ZERO(udp_silent);
  VAR_ZERO(tcp_func);
  VAR_ZERO(tcp_listen_fd);
  VAR_ZERO(tcp_client_fd);
  VAR_ZERO(tcp_client_ip);
  VAR_ZERO(tcp_client_port);
  VAR_ZERO(tcp_silent);

  VAR_ZERO(ircd_func);
  VAR_ZERO(ircd_src_ip);
  VAR_ZERO(ircd_src_port);

  VAR_ZERO(pest_func);
  VAR_ZERO(con_stamp);
  VAR_ZERO(rekey_last);
  VAR_ZERO(ignore_next);
  VAR_ZERO(banner_last);
  VAR_ZERO(addrcast_last);

  BUF_ZERO(gd);
  VAR_ZERO(gd_cnt);
  VAR_ZERO(gd_retry_last);

  BUF_ZERO(hs);
  VAR_ZERO(hs_cnt);

  BUF_ZERO(pen);
  VAR_ZERO(pen_cnt);

  BUF_ZERO(irc_user);
  BUF_ZERO(irc_pass);
  VAR_ZERO(irc_port);
  VAR_ZERO(pest_port);
  BUF_ZERO(scram_pass);

  VAR_ZERO(setport);
  VAR_ZERO(cutoff);
  VAR_ZERO(hstime);
  VAR_ZERO(gdtime);
  VAR_ZERO(gdatt);
  VAR_ZERO(gdint);
  VAR_ZERO(rkrecv);
  VAR_ZERO(rksend);
  VAR_ZERO(rktime);
  VAR_ZERO(rkint);
  VAR_ZERO(igint);
  BUF_ZERO(banner);
  VAR_ZERO(baint);
  BUF_ZERO(acaddr);
  VAR_ZERO(acint);
  VAR_ZERO(irctime);
  BUF_ZERO(master);

  BUF_ZERO(peer);
  VAR_ZERO(peer_cnt);
  VAR_ZERO(peer_id);

  BUF_ZERO(killfile);
  VAR_ZERO(killfile_cnt);

  VAR_ZERO(mess);
  VAR_ZERO(self);
  VAR_ZERO(net);

  BUF_ZERO(dedup_h);
  BUF_ZERO(dedup_t);
  VAR_ZERO(dedup_f);
  VAR_ZERO(dedup_b);

  fputs("state zeroized\n",stderr);
}

void state_rand(void){
  BUF_RAND(data_path);
  BUF_RAND(self_handle);
  VAR_RAND(running);
  VAR_RAND(scram);
  VAR_RAND(mono_offset);
  BUF_RAND(log_path);
  VAR_RAND(log_fp);
  //VAR_RAND(rng_fd);

  BUF_RAND(irc_chan);
  BUF_RAND(irc_host);
  VAR_RAND(irc_conned);
  VAR_RAND(irc_usered);
  VAR_RAND(irc_passed);
  VAR_RAND(irc_nicked);
  VAR_RAND(irc_joined);
  VAR_RAND(irc_ready);

  VAR_RAND(udp_listen_fd);
  VAR_RAND(udp_silent);
  VAR_RAND(tcp_func);
  VAR_RAND(tcp_listen_fd);
  VAR_RAND(tcp_client_fd);
  VAR_RAND(tcp_client_ip);
  VAR_RAND(tcp_client_port);
  VAR_RAND(tcp_silent);

  VAR_RAND(ircd_func);
  VAR_RAND(ircd_src_ip);
  VAR_RAND(ircd_src_port);

  VAR_RAND(pest_func);
  VAR_RAND(con_stamp);
  VAR_RAND(rekey_last);
  VAR_RAND(ignore_next);
  VAR_RAND(banner_last);
  VAR_RAND(addrcast_last);

  BUF_RAND(gd);
  VAR_RAND(gd_cnt);
  VAR_RAND(gd_retry_last);

  BUF_RAND(hs);
  VAR_RAND(hs_cnt);

  BUF_RAND(pen);
  VAR_RAND(pen_cnt);

  BUF_RAND(irc_user);
  BUF_RAND(irc_pass);
  VAR_RAND(irc_port);
  VAR_RAND(pest_port);
  BUF_RAND(scram_pass);

  VAR_RAND(setport);
  VAR_RAND(cutoff);
  VAR_RAND(hstime);
  VAR_RAND(gdtime);
  VAR_RAND(gdatt);
  VAR_RAND(gdint);
  VAR_RAND(rkrecv);
  VAR_RAND(rksend);
  VAR_RAND(rktime);
  VAR_RAND(rkint);
  VAR_RAND(igint);
  BUF_RAND(banner);
  VAR_RAND(baint);
  BUF_RAND(acaddr);
  VAR_RAND(acint);
  VAR_RAND(irctime);
  BUF_RAND(master);

  BUF_RAND(peer);
  VAR_RAND(peer_cnt);
  VAR_RAND(peer_id);

  BUF_RAND(killfile);
  VAR_RAND(killfile_cnt);

  VAR_RAND(mess);
  VAR_RAND(self);
  VAR_RAND(net);

  BUF_RAND(dedup_h);
  BUF_RAND(dedup_t);
  VAR_RAND(dedup_f);
  VAR_RAND(dedup_b);

  fputs("state randomized\n",stderr);
}

void stack_test(void){
  uint8_t stack[STACK_WIPE_SIZE];  
  BUF_ZERO(stack);
}

void stack_zero(void){
  uint8_t stack[STACK_WIPE_SIZE];  
  BUF_ZERO(stack);
  fprintf(stderr,"stack zeroized (%d MB)\n",
          STACK_WIPE_SIZE/MEGABYTE);
}

void stack_rand(void){
  uint8_t stack[STACK_WIPE_SIZE];  
  if(BUF_RAND(stack)){
    fprintf(stderr,"stack randomized (%d MB)\n",
            STACK_WIPE_SIZE/MEGABYTE);
  }else{
    fprintf(stderr,"failed to randomize stack (%d MB)\n",
            STACK_WIPE_SIZE/MEGABYTE);
  }
}

bool file_zero(char const *path){
  FILE       *f;
  struct stat sb;
  size_t      size;
  uint8_t     buf[4096];

  if(stat(path,&sb) != 0){
    return 0;
  }
  size = (size_t)sb.st_size;

  f = fopen(path,"rb+");
  if(!f) return 0;

  BUF_ZERO(buf);
  while(size > 0){
    size_t w_size = size;
    if(w_size > sizeof(buf)){
      w_size = sizeof(buf);
    }

    if(fwrite(buf,1,w_size,f) != w_size){
      fclose(f);
      return 0;
    }

    size -= w_size;
  }

  if(fclose(f) != 0) return 0;
  return 1;
}

bool file_rand(char const *path){
  FILE       *f;
  struct stat sb;
  size_t      size;
  uint8_t     buf[4096];

  if(stat(path,&sb) != 0){
    return 0;
  }
  size = (size_t)sb.st_size;

  f = fopen(path,"rb+");
  if(!f) return 0;

  while(size > 0){
    size_t w_size = size;
    if(w_size > sizeof(buf)){
      w_size = sizeof(buf);
    }

    if(!rng_read(buf,w_size)){
      fclose(f);
      return 0;
    }

    if(fwrite(buf,1,w_size,f) != w_size){
      fclose(f);
      return 0;
    }

    size -= w_size;
  }

  if(fclose(f) != 0) return 0;
  return 1;
}

static int ftw_zero(char const         *path,
                    struct stat const  *sb,
                    int                 flag,
                    struct FTW         *ftw){

  (void)sb;
  (void)ftw;
  if(flag == FTW_F){
    file_zero(path);
  }
  return 0;
}

static int ftw_rand(char const         *path,
                    struct stat const  *sb,
                    int                 flag,
                    struct FTW         *ftw){

  (void)sb;
  (void)ftw;
  if(flag == FTW_F){
    file_rand(path);
  }
  return 0;
}

static bool dir_wipe(char const *path,bool rand){
  return nftw(path,
              rand ? ftw_rand : ftw_zero,
              64,
              FTW_PHYS) == 0;
}

bool dir_zero(char const *path){
  return dir_wipe(path,0);
}

bool dir_rand(char const *path){
  return dir_wipe(path,1);
}

bool rm_nolog(char const *path){
  return remove(path) == 0;
}

static int ftw_rm_rf(char const         *path,
                     struct stat const  *sb,
                     int                 flag,
                     struct FTW         *ftw){

  (void)sb;
  (void)flag;
  (void)ftw;
  rm_nolog(path);
  return 0;
}

bool rm_rf_nolog(char const *path){
  return nftw(path,ftw_rm_rf,64,FTW_DEPTH | FTW_PHYS) == 0;
}

void scram_push(void){
  running   = 0;
  scram     = 1;
  irclog_out("SCRAM ACTIVATED - WIPING STATION STATE");
}

